@@ -26,9 +26,8 @@ class BaseUser(object):
 
 
 class Auth(object):
-    default_next_url = 'homepage'
-    
-    def __init__(self, app, db, user_model=None, prefix='/accounts', name='auth'):
+    def __init__(self, app, db, user_model=None, prefix='/accounts', name='auth',
+                 clear_session=False, default_next_url='/'):
         self.app = app
         self.db = db
         
@@ -37,6 +36,9 @@ class Auth(object):
         self.blueprint = self.get_blueprint(name)
         self.url_prefix = prefix
         
+        self.clear_session = clear_session
+        self.default_next_url = default_next_url
+        
         self.setup()
     
     def get_context_user(self):
@@ -131,7 +133,10 @@ class Auth(object):
         flash('You are logged in as %s' % user.username, 'success')
     
     def logout_user(self, user):
-        session.pop('logged_in', None)
+        if self.clear_session:
+            session.clear()
+        else:
+            session.pop('logged_in', None)
         g.user = None
         flash('You are now logged out', 'success')
     
@@ -160,7 +165,7 @@ class Auth(object):
                     self.login_user(authenticated_user)
                     return redirect(
                         request.args.get('next') or \
-                        url_for(self.default_next_url)
+                        self.default_next_url
                     )
                 else:
                     flash('Incorrect username or password')
@@ -173,7 +178,7 @@ class Auth(object):
         self.logout_user(self.get_logged_in_user())
         return redirect(
             request.args.get('next') or \
-            url_for(self.default_next_url)
+            self.default_next_url
         )
     
     def configure_routes(self):
