@@ -86,14 +86,14 @@ class AdminTestCase(BaseAdminTestCase):
         # check that we have the stuff from the auth module and the index view
         self.assertContext('user', self.admin)
         self.assertContext('model_admins', [
-            admin._registry['amodel'],
-            admin._registry['bdetails'],
-            admin._registry['bmodel'],
-            admin._registry['cmodel'],
-            admin._registry['dmodel'],
-            admin._registry['message'],
-            admin._registry['note'],
-            admin._registry['user'],
+            admin._registry[AModel],
+            admin._registry[BDetails],
+            admin._registry[BModel],
+            admin._registry[CModel],
+            admin._registry[DModel],
+            admin._registry[Message],
+            admin._registry[Note],
+            admin._registry[User],
         ])
         self.assertContext('panels', [
             admin._panels['Notes'],
@@ -112,7 +112,7 @@ class AdminTestCase(BaseAdminTestCase):
             
             # ensure the user, model_admin and form are correct in the context
             self.assertContext('user', self.admin)
-            self.assertContext('model_admin', admin._registry['user'])
+            self.assertContext('model_admin', admin._registry[User])
             
             self.assertTrue('form' in self.flask_app._template_context)
             frm = self.flask_app._template_context['form']
@@ -189,7 +189,7 @@ class AdminTestCase(BaseAdminTestCase):
             
             # check the user, model_admin and form are correct in the context
             self.assertContext('user', self.admin)
-            self.assertContext('model_admin', admin._registry['user'])
+            self.assertContext('model_admin', admin._registry[User])
             
             self.assertTrue('form' in self.flask_app._template_context)
             frm = self.flask_app._template_context['form']
@@ -292,7 +292,7 @@ class AdminTestCase(BaseAdminTestCase):
             self.assertEqual(resp.status_code, 200)
             
             self.assertContext('user', self.admin)
-            self.assertContext('model_admin', admin._registry['user'])
+            self.assertContext('model_admin', admin._registry[User])
             
             query = self.get_context('query')
             self.assertEqual(list(query), [])
@@ -330,6 +330,102 @@ class AdminTestCase(BaseAdminTestCase):
             
             self.assertEqual(User.select().count(), 0)
     
+    def test_model_admin_recursive_delete(self):
+        self.create_users()
+        
+        m1 = Message.create(user=self.normal, content='test1')
+        m2 = Message.create(user=self.normal, content='test2')
+        m3 = Message.create(user=self.admin, content='test3')
+        
+        n1 = Note.create(user=self.normal, message='test1')
+        n2 = Note.create(user=self.normal, message='test2')
+        n3 = Note.create(user=self.admin, message='test3')
+        
+        a1 = AModel.create(a_field='a1')
+        a2 = AModel.create(a_field='a2')
+        b1 = BModel.create(b_field='b1', a=a1)
+        b2 = BModel.create(b_field='b2', a=a2)
+        bd1= BDetails.create(b=b1)
+        bd2= BDetails.create(b=b2)
+        c1 = CModel.create(c_field='c1', b=b1)
+        c2 = CModel.create(c_field='c2', b=b2)
+        d1 = DModel.create(d_field='d1', c=c1)
+        d2 = DModel.create(d_field='d2', c=c2)
+        
+        with self.flask_app.test_client() as c:
+            self.login(c)
+            
+            resp = c.get('/admin/amodel/delete/?id=%d' % (a1.id))
+            self.assertEqual(resp.status_code, 200)
+            
+            collected = self.get_context('collected')
+            self.assertEqual(collected, {
+                a1.id: [
+                    (1, BModel, 'a', [b1]),
+                    (2, BDetails, 'a', [bd1]),
+                    (2, CModel, 'a', [c1]),
+                    (3, DModel, 'a', [d1]),
+                ]
+            })
+            
+            resp = c.post('/admin/amodel/delete/', data={'id': a1.id})
+            self.assertEqual(resp.status_code, 302)
+            self.assertEqual(AModel.select().count(), 1)
+            self.assertEqual(BModel.select().count(), 1)
+            self.assertEqual(BDetails.select().count(), 1)
+            self.assertEqual(CModel.select().count(), 1)
+            self.assertEqual(DModel.select().count(), 1)
+            
+            # send it a single id
+            resp = c.get('/admin/user/delete/?id=%d' % (self.normal.id))
+            self.assertEqual(resp.status_code, 200)
+            
+            query = self.get_context('query')
+            self.assertEqual(list(query), [self.normal])
+            
+            collected = self.get_context('collected')
+            self.assertEqual(len(collected), 1)
+            u_k = collected[self.normal.id]
+            self.assertEqual(len(u_k), 2)
+
+            self.assertEqual(u_k, [
+                (1, Message, 'user', [m1, m2]),
+                (1, Note, 'user', [n1, n2]),
+            ])
+            
+            # post to it, get a redirect on success
+            resp = c.post('/admin/user/delete/', data={'id': self.normal.id})
+            self.assertEqual(resp.status_code, 302)
+            
+            self.assertEqual(User.select().count(), 2)
+            self.assertEqual(Message.select().count(), 1)
+            self.assertEqual(Note.select().count(), 1)
+            
+            resp = c.get('/admin/user/delete/?id=%d&id=%d' % (self.admin.id, self.inactive.id))
+            self.assertEqual(resp.status_code, 200)
+            
+            collected = self.get_context('collected')
+
+            self.assertEqual(len(collected), 2)
+            u_k = collected[self.admin.id]
+            self.assertEqual(len(u_k), 2)
+
+            self.assertEqual(u_k, [
+                (1, Message, 'user', [m3]),
+                (1, Note, 'user', [n3]),
+            ])
+            
+            u_k = collected[self.inactive.id]
+            self.assertEqual(len(u_k), 0)
+            
+            # post to it, get a redirect on success
+            resp = c.post('/admin/user/delete/', data={'id': [self.admin.id, self.inactive.id]})
+            self.assertEqual(resp.status_code, 302)
+            
+            self.assertEqual(User.select().count(), 0)
+            self.assertEqual(Message.select().count(), 0)
+            self.assertEqual(Note.select().count(), 0)
+    
     def test_model_admin_index(self):
         self.create_users()
         
@@ -340,7 +436,7 @@ class AdminTestCase(BaseAdminTestCase):
             self.assertEqual(resp.status_code, 200)
             
             self.assertContext('user', self.admin)
-            self.assertContext('model_admin', admin._registry['user'])
+            self.assertContext('model_admin', admin._registry[User])
             self.assertContext('ordering', 'username')
             
             query_filter = self.get_context('query_filter')
@@ -373,7 +469,7 @@ class AdminTestCase(BaseAdminTestCase):
             self.assertEqual(resp.status_code, 200)
             
             self.assertContext('user', self.admin)
-            self.assertContext('model_admin', admin._registry['user'])
+            self.assertContext('model_admin', admin._registry[User])
             self.assertContext('ordering', '')
 
             query = self.get_context('query')
@@ -411,19 +507,19 @@ class AdminTestCase(BaseAdminTestCase):
             ])
             
             # test a lookup spanning a relation
-            resp = c.get('/admin/note/?user_id__eq=%d' % self.normal.id)
+            resp = c.get('/admin/note/?user__eq=%d' % self.normal.id)
             self.assertEqual(resp.status_code, 200)
             
-            self.assertContext('model_admin', admin._registry['note'])
+            self.assertContext('model_admin', admin._registry[Note])
             
             query = self.get_context('query')
             self.assertEqual(list(query.get_list()), notes[self.normal])
             
             # test a multi-value lookup spanning a relation
-            resp = c.get('/admin/note/?user_id__in=%d&user_id__in=%d' % (self.normal.id, self.admin.id))
+            resp = c.get('/admin/note/?user__in=%d&user__in=%d' % (self.normal.id, self.admin.id))
             self.assertEqual(resp.status_code, 200)
             
-            self.assertContext('model_admin', admin._registry['note'])
+            self.assertContext('model_admin', admin._registry[Note])
             
             query = self.get_context('query')
             expected_notes = notes[self.admin] + notes[self.normal]
@@ -637,12 +733,12 @@ class TemplateHelperTestCase(FlaskPeeweeTestCase):
     
     def test_get_model_admins(self):
         self.assertEqual(self.th.get_model_admins(), {'model_admins': [
-            admin._registry['amodel'],
-            admin._registry['bdetails'],
-            admin._registry['bmodel'],
-            admin._registry['cmodel'],
-            admin._registry['dmodel'],
-            admin._registry['message'],
-            admin._registry['note'],
-            admin._registry['user'],
+            admin._registry[AModel],
+            admin._registry[BDetails],
+            admin._registry[BModel],
+            admin._registry[CModel],
+            admin._registry[DModel],
+            admin._registry[Message],
+            admin._registry[Note],
+            admin._registry[User],
         ]})
